import Movable from "./Movable";

export default class MovableManager {
    private _stage: FYGE.Stage;
    private _movableList: Movable[] = [];
    constructor(stage: FYGE.Stage) {
        this._stage = stage;
        this._stage.addEventListener(FYGE.Event.ENTER_FRAME, this.onEnterFrame);
    }

    onEnterFrame = () => {
        this.step();
        this.checkRemove();
        console.log('移动对象的数量:',this._movableList.length)
    }

    private calcCanRemove(item: FYGE.DisplayObject) {
        if (item.y > 1624) return true;
        if (item.y < -item.height) return true;
        if (item.x > 750) return true;
        if (item.x < -item.width) return true;
        return false;
    }

    private checkRemove() {
        for (let index = 0; index < this._movableList.length; index++) {
            const item = this._movableList[index];
            if (this.calcCanRemove(item)) {
                this._movableList.splice(index, 1);
                item.parent && item.parent.removeChild(item);
                index--;
            }
        }
    }

    private step() {
        this._movableList.forEach(item => item.step());
    }

    add(item: Movable) {
        this._movableList.push(item);
    }

    remove(item: Movable) {
        const index = this._movableList.indexOf(item);
        this._movableList.splice(index, 1);
    }

}